class AuthD::Request
	IPC::JSON.message ValidateUser, 2 do
		property user           : UserID
		property activation_key : String

		def initialize(@user, @activation_key)
		end

		def handle(authd : AuthD::Service, fd : Int32)
			user = authd.user? @user
			return Response::Error.new "user not found" if user.nil?

			if user.contact.activation_key.nil?
				return Response::Error.new "user already validated"
			end

			# Remove the user contact activation key: the email is validated.
			if user.contact.activation_key == @activation_key
				user.contact.activation_key = nil
			else
				return Response::Error.new "wrong activation key"
			end

			authd.users_per_uid.update user.uid.to_s, user

			Response::UserValidated.new user.to_public
		end
	end
	AuthD.requests << ValidateUser

	IPC::JSON.message GetUser, 3 do
		property user : UserID

		def initialize(@user)
		end

		def handle(authd : AuthD::Service, fd : Int32)
			user = authd.user? @user
			return Response::Error.new "user not found" if user.nil?

			Response::User.new user.to_public
		end
	end
	AuthD.requests << GetUser

	IPC::JSON.message GetUserByCredentials, 4 do
		property login      : String
		property password   : String

		def initialize(@login, @password)
		end

		def handle(authd : AuthD::Service, fd : Int32)
			user = authd.users_per_login.get? @login
			return Response::Error.new "invalid credentials" unless user

			if authd.hash_password(@password) != user.password_hash
				return Response::Error.new "invalid credentials"
			end

			user.date_last_connection = Time.local

			# Change the date of the last connection.
			authd.users_per_uid.update user.uid.to_s, user

			Response::User.new user.to_public
		end
	end
	AuthD.requests << GetUserByCredentials
end
